unit SEQUENCE;

{ By Michael Taylor 76625,2552
  This Unit implements a VCL TTable descendant somewhat like Oracle's
  Sequences.  To use, install in your Component Library and drop onto
  your form.  You will need to create a table to house the sequences.
  The table should have one sequence per column, but can have any number
  of columns of type Long Integer (if paradox is used).  For example, I
  created a table like:
     SEQUENCE.DB
        BOOK_ID     Long Integer
        CONTACT_ID  Long Integer

  On the VCL properties, specify the table as your table name with Active
  set to False and the sequence as the name of the sequence (column) you
  wish to use.  You can have more than one Sequence component on the same
  form or can dynamically change the Sequence name in your code.

  The Method NextVal will reserve and return the next available sequence
  number for your column.  The CurrVal returns the most recently acquired
  sequence number for the column.  This is my first VCL and may not be
  completely bullet-proof, but I haven't run into any problems with it yet. }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables;

type
  TSequence = class(TTable)
  private
    { Private declarations }
    SeqName: string;        { Name of current sequence }
    SeqRsvCount: longint;   { Not used yet }
    SeqField: TField;       { Used internally }
  protected
    { Protected declarations }
    function CreateValue: longint; virtual;
  public
    { Public declarations }
    function NextVal: longint; virtual;
    function CurrVal: longint; virtual;
  published
    { Published declarations }
    property SequenceName: string read SeqName write SeqName;
  end;

  SeqFieldNotFound = class(Exception);

procedure Register;

implementation

function TSequence.CreateValue: longint;
var
  SeqValue: longint;
begin
  Open;  {Open the Sequence Table}
  Edit;  {Locks the Sequence Table}
  SeqField := FindField(SeqName);
  if SeqField = nil then
    raise SeqFieldNotFound.Create('Sequence Name Not Found');
  SeqValue := SeqField.AsInteger + 1;
  TIntegerField(SeqField).Value := SeqValue;
  Post;  {Post new value to Sequence Table}
  Close; {Close the Sequence Table}
  CreateValue := SeqValue;
end;

function TSequence.NextVal: longint;
begin
  NextVal := CreateValue;
end;

function TSequence.CurrVal: longint;
begin
  Open;  {Open the Sequence Table}
  SeqField := FindField(SeqName);
  if SeqField = nil then
    raise SeqFieldNotFound.Create('Sequence Not Found');
  Close; {Close the Sequence Table}
  CurrVal := SeqField.AsInteger;
end;

procedure Register;
begin
  RegisterComponents('Data Access', [TSequence]);
end;

end.
